import os

import pytest

from src.data_handling.data_generator import DataGenerator


class TestDataGenerator:

    # @pytest.fixture(autouse=True, scope='module')
    # def teardown_module(self):
    #     yield
    #     if "data" in os.listdir(os.path.dirname(__file__)):
    #         shutil.rmtree(os.path.join(os.path.dirname(__file__), "data"), ignore_errors=True)

    @pytest.fixture
    def gen(self):
        return DataGenerator(os.path.join(os.path.dirname(__file__), 'data'), 'AIRBASE', 'DEBW107', ['o3', 'temp'],
                             'datetime', 'variables', 'o3')

    def test_init(self, gen):
        assert gen.data_path == os.path.join(os.path.dirname(__file__), 'data')
        assert gen.network == 'AIRBASE'
        assert gen.stations == ['DEBW107']
        assert gen.variables == ['o3', 'temp']
        assert gen.station_type is None
        assert gen.interpolate_dim == 'datetime'
        assert gen.target_dim == 'variables'
        assert gen.target_var == 'o3'
        assert gen.interpolate_method == "linear"
        assert gen.limit_nan_fill == 1
        assert gen.window_history_size == 7
        assert gen.window_lead_time == 4
        assert gen.transform_method == "standardise"
        assert gen.kwargs == {}

    def test_repr(self, gen):
        path = os.path.join(os.path.dirname(__file__), 'data')
        assert gen.__repr__().rstrip() == f"DataGenerator(path='{path}', network='AIRBASE', stations=['DEBW107'], "\
                                          f"variables=['o3', 'temp'], station_type=None, interpolate_dim='datetime', " \
                                          f"target_dim='variables', target_var='o3', **{{}})".rstrip()

    def test_len(self, gen):
        assert len(gen) == 1
        gen.stations = ['station1', 'station2', 'station3']
        assert len(gen) == 3

    def test_getitem(self, gen):
        gen.kwargs = {'statistics_per_var': {'o3': 'dma8eu', 'temp': 'maximum'}}
        station = gen["DEBW107"]
        assert len(station) == 2
        assert station[0].Stations.data == "DEBW107"
        assert station[0].data.shape[1:] == (8, 1, 2)
        assert station[1].data.shape[-1] == gen.window_lead_time
        assert station[0].data.shape[1] == gen.window_history_size + 1

    def test_iter(self, gen):
        assert hasattr(gen, '_iterator') is False
        iter(gen)
        assert hasattr(gen, '_iterator')
        assert gen._iterator == 0

    def test_next(self, gen):
        gen.kwargs = {'statistics_per_var': {'o3': 'dma8eu', 'temp': 'maximum'}}
        for i, d in enumerate(gen, start=1):
            assert i == gen._iterator

    def test_get_station_key(self, gen):
        gen.stations.append("DEBW108")
        f = gen.get_station_key
        iter(gen)
        assert f(None) == "DEBW107"
        with pytest.raises(KeyError) as e:
            f([None, None])
        assert "More than one key was given: [None, None]" in e.value.args[0]
        assert f(1) == "DEBW108"
        assert f([1]) == "DEBW108"
        with pytest.raises(KeyError) as e:
            f(3)
        assert "3 is not in range(0, 2)" in e.value.args[0]
        assert f("DEBW107") == "DEBW107"
        assert f(["DEBW108"]) == "DEBW108"
        with pytest.raises(KeyError) as e:
            f("DEBW999")
        assert "DEBW999 is not in stations" in e.value.args[0]
        with pytest.raises(KeyError) as e:
            f(6.5)
        assert "key has to be from Union[str, int]. Given was 6.5 (float)"
